@@ -1,7 +1,7 @@
 #
 # Run this Tcl script to generate the pragma.html file.
 #
-set rcsid {$Id: pragma.tcl,v 1.28 2007/08/28 08:19:49 danielk1977 Exp $}
+set rcsid {$Id: pragma.tcl,v 1.19 2007/01/27 02:24:57 drh Exp $}
 source common.tcl
 header {Pragma statements supported by SQLite}
 
@@ -27,21 +27,21 @@ different in the following important respects:
     Unknown pragmas are simply ignored. This means if there is a typo in 
     a pragma statement the library does not inform the user of the fact.
 <li>Some pragmas take effect during the SQL compilation stage, not the
-    execution stage. This means if using the C-language sqlite3_prepare(), 
+    execution stage. This means if using the C-language sqlite3_compile(), 
     sqlite3_step(), sqlite3_finalize() API (or similar in a wrapper 
     interface), the pragma may be applied to the library during the 
-    sqlite3_prepare() call.
+    sqlite3_compile() call.
 <li>The pragma command is unlikely to be compatible with any other SQL
     engine.
 </ul>
 
 <p>The available pragmas fall into four basic categories:</p>
 <ul>
+<li>Pragmas used to <a href="#schema">query the schema</a> of the current 
+    database.
 <li>Pragmas used to <a href="#modify">modify the operation</a> of the 
     SQLite library in some manner, or to query for the current mode of 
     operation.
-<li>Pragmas used to <a href="#schema">query the schema</a> of the current 
-    database.
 <li>Pragmas used to <a href="#version">query or modify the databases two 
     version values</a>, the schema-version and the user-version.
 <li>Pragmas used to <a href="#debug">debug the library</a> and verify that
@@ -71,46 +71,26 @@ Section {Pragmas to modify library operation} modify
 puts {
 <ul>
 <a name="pragma_auto_vacuum"></a>
-<li><p><b>PRAGMA auto_vacuum;<br>
-          PRAGMA auto_vacuum = </b>
-            <i>0 | none | 1 | full | 2 | incremental</i><b>;</b></p>
-    <p>Query or set the auto-vacuum flag in the database.</p>
+<li><p><b>PRAGMA auto_vacuum;
+       <br>PRAGMA auto_vacuum = </b><i>0 | 1</i><b>;</b></p>
+    <p> Query or set the auto-vacuum flag in the database.</p>
 
-    <p>Normally, (that is to say when auto_vacuum is 0 or "none")
-    when a transaction that deletes data from a database is
+    <p>Normally, when a transaction that deletes data from a database is
     committed, the database file remains the same size. Unused database file 
-    pages are added to a "freelist" are reused for subsequent inserts.  The
-    database file does not shrink.
-    In this mode the <a href="lang_vacuum.html">VACUUM</a>
-    command can be used to reclaim unused space.</p>
+    pages are marked as such and reused later on, when data is inserted into 
+    the database. In this mode the <a href="lang_vacuum.html">VACUUM</a>
+    command is used to reclaim unused space.</p>
 
-    <p>When the auto-vacuum flag is 1 (full), the freelist pages are
-    moved to the end of the file and the file is truncated to remove
-    the freelist pages at every commit.
-    Note, however, that auto-vacuum only truncates the freelist pages
-    from the file.  Auto-vacuum does not defragment the database nor
-    repack individual database pages the way that the
-    <a href="lang_vacuum.html">VACUUM</a> command does.  In fact, because
-    it moves pages around within the file, auto-vacuum can actually
-    make fragmentation worse.</p>
+    <p>When the auto-vacuum flag is set, the database file shrinks when a
+    transaction that deletes data is committed (The VACUUM command is not
+    useful in a database with the auto-vacuum flag set). To support this
+    functionality the database stores extra information internally, resulting
+    in slightly larger database files than would otherwise be possible.</p>
 
-    <p>Auto-vacuuming is only possible if the database stores some
-    additional information that allows each database page to be
-    traced backwards to its referer.  Therefore, auto-vacuuming must
-    be turned on before any tables are created.  It is not possible
-    to enable or disable auto-vacuum after a table has been created.</p>
-
-    <p>When the value of auto-vacuum is 2 (incremental) then the additional
-    information needed to do autovacuuming is stored in the database file
-    but autovacuuming does not occur automatically at each commit as it
-    does with auto_vacuum==full.  In incremental mode, the separate
-    <a href="#pragma_incremental_vacuum">incremental_vacuum</a> pragma must
-    be invoked to cause the vacuum to occur.</p>
-
-    <p>The database connection can be changed between full and incremental
-    autovacuum mode at will.  However, the connection cannot be changed
-    in and out of the "none" mode after any table has been created in the
-    database.
+    <p>It is only possible to modify the value of the auto-vacuum flag before
+    any tables have been created in the database. No error message is 
+    returned if an attempt to modify the auto-vacuum flag is made after
+    one or more tables have been created.
     </p></li>
 
 <a name="pragma_cache_size"></a>
@@ -241,28 +221,6 @@ puts {
     </p>
 </li>
 
-<a name="pragma_incremental_vacuum"></a>
-<li><p><b>PRAGMA incremental_vacuum</b><i>(N)</i><b>;</b></p>
-    <p>The incremental_vacuum pragma causes up to <i>N</i> pages to
-    be removed from the freelist.  The database file is truncated by
-    the same amount.  The incremental_vacuum pragma has no effect if
-    the database is not in
-    <a href="#pragma_auto_vacuum">auto_vacuum==incremental</a> mode
-    or if there are no pages on the freelist.  If there are fewer than
-    <i>N</i> pages on the freelist, then the entire freelist is cleared.</p>
-
-    <p>As of version 3.4.0 (the first version that supports
-    incremental_vacuum) this feature is still experimental.  Possible
-    future changes include enhancing incremental vacuum to do
-    defragmentation and node repacking just as the full-blown
-    <a href="lang_vacuum.html">VACUUM</a> command does.  And
-    incremental vacuum may be promoted from a pragma to a separate
-    SQL command, or perhaps some variation on the VACUUM command.
-    Programmers are cautioned to not become enamored with the
-    current syntax or functionality as it is likely to change.</p>
-</li>
-
-
 <a name="pragma_legacy_file_format"></a>
 <li><p><b>PRAGMA legacy_file_format;
        <br>PRAGMA legacy_file_format = <i>ON | OFF</i></b></p>
@@ -270,58 +228,13 @@ puts {
     flag.  When this flag is on, new SQLite databases are created in
     a file format that is readable and writable by all versions of
     SQLite going back to 3.0.0.  When the flag is off, new databases
-    are created using the latest file format which might not be
+    are created using the latest file format which might to be
     readable or writable by older versions of SQLite.</p>
 
-    <p>This flag only affects newly created databases.  It has no
-    effect on databases that already exist.</p>
+    <p>This flag only effects newly created databases.  It has no
+    effect on databases that already exists.</p>
 </li>
 
-<a name="pragma_locking_mode"></a>
-<li><p><b>PRAGMA locking_mode;
-       <br>PRAGMA locking_mode = <i>NORMAL | EXCLUSIVE</i></b></p>
-    <p>This pragma sets or queries the database connection locking-mode. 
-    The locking-mode is either NORMAL or EXCLUSIVE.
-
-    <p>In NORMAL locking-mode (the default), a database connection
-    unlocks the database file at the conclusion of each read or
-    write transaction. When the locking-mode is set to EXCLUSIVE, the
-    database connection never releases file-locks. The first time the
-    database is read in EXCLUSIVE mode, a shared lock is obtained and 
-    held. The first time the database is written, an exclusive lock is
-    obtained and held.</p>
-
-    <p>Database locks obtained by a connection in EXCLUSIVE mode may be
-    released either by closing the database connection, or by setting the
-    locking-mode back to NORMAL using this pragma and then accessing the
-    database file (for read or write). Simply setting the locking-mode to
-    NORMAL is not enough - locks are not be released until the next time
-    the database file is accessed.</p>
-
-    <p>There are two reasons to set the locking-mode to EXCLUSIVE. One
-    is if the application actually wants to prevent other processes from
-    accessing the database file. The other is that a small number of
-    filesystem operations are saved by optimizations enabled in this
-    mode. This may be significant in embedded environments.</p>
-
-    <p>When the locking_mode pragma specifies a particular database,
-    for example:</p>
-
-    <blockquote>
-PRAGMA <b>main.</b>locking_mode=EXCLUSIVE;
-    </blockquote>
-
-    <p>Then the locking mode applies only to the named database.  If no
-    database name qualifier preceeds the "locking_mode" keyword then
-    the locking mode is applied to all databases, including any new
-    databases added by subsequent <a href="lang_attach.html">ATTACH</a>
-    commands.</p>
-
-   <p>The "temp" database (in which TEMP tables and indices are stored)
-   always uses exclusive locking mode.  The locking mode of temp cannot
-   be changed.  All other databases use the normal locking mode by default
-   and are affected by this pragma.</p>
-</li>
 
 <a name="pragma_page_size"></a>
 <li><p><b>PRAGMA page_size;
@@ -335,16 +248,6 @@ PRAGMA <b>main.</b>locking_mode=EXCLUSIVE;
     </p>
 </li>
 
-<a name="pragma_max_page_count"></a>
-<li><p><b>PRAGMA max_page_count;
-       <br>PRAGMA max_page_count = </b><i>N</i><b>;</b></p>
-    <p>Query or set the maximum number of pages in the database file.
-    Both forms of the pragma return the maximum page count.  The second
-    form attempts to modify the maximum page count.  The maximum page
-    count cannot be reduced below the current database size.
-    </p>
-</li>
-
 <a name="pragma_read_uncommitted"></a>
 <li><p><b>PRAGMA read_uncommitted;
        <br>PRAGMA read_uncommitted = </b><i>0 | 1</i><b>;</b></p>
@@ -522,13 +425,6 @@ puts {
     foreign key. The callback function will be invoked once for each
     column in each foreign key.</p></li>
 
-<a name="pragma_freelist_count"></a>
-<li><p><b>PRAGMA [database].freelist_count;</b></p>
-    <p>Return the number of unused pages in the database file. Running
-    a <a href="#pragma_incremental_vacuum">"PRAGMA incremental_vaccum(N);"</a> 
-    command with a large value of N will shrink the database file by this
-    number of pages. </p></li>
-
 <a name="pragma_index_info"></a>
 <li><p><b>PRAGMA index_info(</b><i>index-name</i><b>);</b></p>
     <p>For each column that the named index references, invoke the 
